/*
 * pi - P user Interface
 * 
 * $Id: utils.h,v 1.1 2001/02/05 15:51:08 malekith Exp $
 * Author: Michal Moskal <malekith@pld.org.pl>
 * include COPYING-GNU
 */

#ifndef _PI_UTILS_H
#define _PI_UTILS_H

#include <assert.h>

/* logic.c */
int pi_key_pressed(struct pi_object *ctx, int key);
int pi_object_key(struct pi_object *ctx, int key);
struct pi_object *pi_add_child(struct pi_object *ctx, struct pi_object *o);
void pi_kill(struct pi_object *ctx);
void pi_add_window(struct pi_object *p);
int pi_push_event(void);
int pi_exec(struct pi_object *win);
void pi_repaint(void);
struct pi_object *pi_new_object(struct pi_object *ctx, struct pi_object *super);

/* util.c */
void pi_draw(struct pi_object *ctx);
void pi_send_cc(struct pi_object *ctx, int child_command);
void pi_center(struct pi_object *w);
void pi_locate(struct pi_object *self, int *xx, int *yy);
void pi_setstr(char **p, const char *s);
int pi_is_active(struct pi_object *self);
void pi_setpos(struct pi_object *p, int x, int y, int w, int h);
void pi_scrollbar(int i, int top, int h, int nlines, int color, int color_hl);
void pi_init_size(struct pi_object *ret, int x, int y, int w, int h);
void pi_set_color(struct pi_object *ctx, int hl);

#define pi_new_init(tags, sup) 					\
	struct pi_object *ret;					\
	static struct pi_object *super_object;			\
								\
	if (super_object == 0)					\
		super_object = pi_add_object tags ;		\
								\
	if (super)						\
		pi_set_tag(super, pi_tag_super, super_object);	\
	else							\
		super = super_object;				\
								\
	ret = pi_new_ ## sup(ctx, super)

#define pi_add(win, what, tags) \
	pi_add_child(win, pi_new_ ## what (pi_add_object tags , 0))

#define pi_copy_int(attr) \
	if (pi_exists(ctx, pi_tag_ ## attr)) \
		pi_set_tag(ret, pi_tag_ ## attr , pi_int(attr))

#define pi_copy_ptr(attr) \
	if (pi_exists(ctx, pi_tag_ ## attr)) \
		pi_set_tag(ret, pi_tag_ ## attr , pi_ptr(attr))

/* wordbreak.c */
struct pi_wordbreak_res {
	char **lines;
	int nlines;
	int width;
};

/* try to break words in optimal way, width is maximal allowed width. 
 * result needs to be free()ed (just once, free()ing lines is
 * not needed ... )
 */
struct pi_wordbreak_res *pi_wordbreak(const char *string, int width);

#endif /* _PI_UTILS_H */
