/*
   $Id: srcget.c,v 1.7 2001/10/14 08:42:31 pawelk Exp $

   Author: Pawe Koodziej <pawelk@pld.org.pl>
 */

#define FILE_NAME 1000000
#define FILE_SIZE 1000001
#define no_scan 0

#include "rpmmenlib.h"
#include "pkgs_source.h"
#include "../other/misc.h"

/*
   #define TEST_SRCGET
 */


/* addpackage - add package to TPkgSet structure
   ARGS: pkgs  - structure to which I will add new package
   fname - file name of package
   h     - pointer to rpm-header, or NULL
   RET : none
 */
void addPackage(TPkgSet * pkgs, char *fname, Header * h,int ps_i)
{
	TPkgInfo *pk;
	int count, type;
	pk = (TPkgInfo *) malloc(sizeof(TPkgInfo));

	if (h != NULL) {
		memcpy(&(pk->h), h, sizeof(Header));
	}
	pk->flags = 0;
	pk->selected = 0;
	pk->neededBy = NULL;
	pk->ps_i=ps_i;

	if (h == NULL) {
		pk->name = strdup(fname);

	} else {
		char *tmp2;

		pk->flags |= TPKGINF_HASHDR;
		headerGetEntry(*h, RPMTAG_NAME, &type, (void *) &tmp2, &count);
		pk->name = strdup(tmp2);
	}

	pk->fname = strdup(fname);
	n_array_push(pkgs->packages, pk);
	log_message("%s: %s", __FUNCTION__, fname);
}

/** 
 * \param local_tocfile if not NULL use this local file instead of tocfile from    source medium
 * \return NULL when error */

TPkgSet *getPackagesList(char *local_tocfile)
{
	TPkgSet *pkgs;
	Header h;
	int fd;
	int end;

	char *name1;
	int type, count;
	FD_t fdt;
	int ps_i;
	FILE *fmd5;
	char buf[1000], md5[100], fname[1000];
		
	pkgs = (TPkgSet *) malloc(sizeof(TPkgSet));
	pkgs->packages = pkgs_array_new();

	for(ps_i=0;ps_i<PsSet.count;ps_i++){
		fd=psOpenInfo(&(PsSet.ps[ps_i]),"tocfile.md5");
		if(fd<0){
			log_message(_("Can't open file tocfile.md5 (%s)"),
					strerror(errno));
			return NULL;
		}
		log_debug("tocfile.md5 opened as fd");
		fmd5=fdopen(fd,"rt");
		if(!fmd5){
			log_message(_("Can't do fdopen on tocfile.md5"));
			return NULL;
		}
		log_debug("md5 file opened as FILE");
		/* made tocfiles up to date */
		while(!feof(fmd5)){
			fgets(buf,999,fmd5);
			if(strlen(buf)<30){
				log_message("Short line in tocfile.md5");
				continue;
			}
			sscanf(buf,"%s  %s",md5,fname);
			fd=psOpenInfo(&(PsSet.ps[ps_i]),fname);
			if(fd==-1){
				log_message("refetching file %s",fname);
				psCacheRemoveInfo(&(PsSet.ps[ps_i]),fname);
				fd=psOpenInfo(&(PsSet.ps[ps_i]),fname);
			}
			if(strcmp(md5,getMD5File(fd))!=0){
				log_message("File %s was out of date",fname);
				psCacheRemoveInfo(&(PsSet.ps[ps_i]),fname);
			}
			close(fd);
		}
		log_message("tocfiles updated");
		/* read tocfiles */
		fseek(fmd5,0,SEEK_SET);
		while(!feof(fmd5)){
			fgets(buf,999,fmd5);
			if(strlen(buf)<30){
				log_message("Short line in tocfile.md5");
				continue;
			}
			sscanf(buf,"%s  %s",md5,fname);

			fd=psOpenInfo(&(PsSet.ps[ps_i]),fname);

			if (fd < 0) {
				log_message(_("Error opening file %s (%s)"),
						fname,strerror(errno));
				return NULL;
			}	

			fdt = fdNew("");
			fdt = fdDup(fd);
			end = lseek(fd, 0, SEEK_END);
			lseek(fd, 0, SEEK_SET);

			while (end > lseek(fd, 0, SEEK_CUR)) {

				h = headerRead(fdt, HEADER_MAGIC_YES);
				if (!h) {
					log_message(_("Error reading header "
							"from file: %s"),
							RPMMEN_TOCFILE);
					close(fd);
					break;	/* stop reading headers file */
				}
				/*read(fd, &size1, sizeof(int)); */
				headerGetEntry(h, FILE_NAME, &type,
						(void **) &(name1), &count);

				addPackage(pkgs, name1, h ? &h : NULL, ps_i);

			}

			fdFree(fdt, "");
			close(fd);
		}
		fclose(fmd5);
	}
	return pkgs;
}

